@@ -167,9 +167,9 @@ const unsigned short codepages[38][256] =
      112,   113,   114,   115,   116,   117,   118,   119,   120,   121,   122,   123,   124,   125,   126,   127,
   0x0402,0x0403,0x201A,0x0453,0x201E,0x2026,0x2020,0x2021,0x20AC,0x2030,0x0409,0x2039,0x040A,0x040C,0x040B,0x040F,
   0x0452,0x2018,0x2019,0x201C,0x201D,0x2022,0x2013,0x2014,   152,0x2122,0x0459,0x203A,0x045A,0x045C,0x045B,0x045F,
-     160,0x040E,0x045E,0x0408,   164,0x0409,   166,   167,0x0401,   169,0x0404,   171,   172,   173,   174,0x0407,
+     160,0x040E,0x045E,0x0408,   164,0x0490,   166,   167,0x0401,   169,0x0404,   171,   172,   173,   174,0x0407,
      176,   177,0x0406,0x0456,0x0491,   181,   182,   183,0x0451,0x2116,0x0454,   187,0x0458,0x0405,0x0455,0x0457,
-  0x0100,0x0411,0x0412,0x0413,0x0414,0x0415,0x0416,0x0417,0x0418,0x0419,0x041A,0x041B,0x041C,0x041D,0x041E,0x041F,
+  0x0410,0x0411,0x0412,0x0413,0x0414,0x0415,0x0416,0x0417,0x0418,0x0419,0x041A,0x041B,0x041C,0x041D,0x041E,0x041F,
   0x0420,0x0421,0x0422,0x0423,0x0424,0x0425,0x0426,0x0427,0x0428,0x0429,0x042A,0x042B,0x042C,0x042D,0x042E,0x042F,
   0x0430,0x0431,0x0432,0x0433,0x0434,0x0435,0x0436,0x0437,0x0438,0x0439,0x043A,0x043B,0x043C,0x043D,0x043E,0x043F,
   0x0440,0x0441,0x0442,0x0443,0x0444,0x0445,0x0446,0x0447,0x0448,0x0449,0x044A,0x044B,0x044C,0x044D,0x044E,0x044F
@@ -684,7 +684,7 @@ void Input::file_init(file_encoding_type enc)
           ulen_ = 2;
           if (utf8_[0] == '\0' && utf8_[1] == '\0')  // UTF-32 big endian BOM 0000XXXX?
           {
-            if (::fread(&utf8_[2], 2, 1, file_) == 1)
+            if (::fread(utf8_ + 2, 2, 1, file_) == 1)
             {
               ulen_ = 4;
               if (utf8_[2] == '\xfe' && utf8_[3] == '\xff') // UTF-32 big endian BOM 0000FEFF?
@@ -703,26 +703,41 @@ void Input::file_init(file_encoding_type enc)
           }
           else if (utf8_[0] == '\xff' && utf8_[1] == '\xfe') // UTF-16 or UTF-32 little endian BOM FFFEXXXX?
           {
-            if (::fread(&utf8_[2], 2, 1, file_) == 1)
+            if (::fread(utf8_ + 2, 2, 1, file_) == 1)
             {
+              size_ = 0;
               if (utf8_[2] == '\0' && utf8_[3] == '\0') // UTF-32 little endian BOM FFFE0000?
               {
-                size_ = 0;
                 ulen_ = 0;
                 utfx_ = file_encoding::utf32le;
               }
               else
               {
-                size_ = 0;
-                uidx_ = 2;
-                ulen_ = 1;
+                int c = static_cast<unsigned char>(utf8_[2]) | static_cast<unsigned char>(utf8_[3]) << 8;
+                if (c < 0x80)
+                {
+                  uidx_ = 2;
+                  ulen_ = 1;
+                }
+                else
+                {
+                  if (c >= 0xD800 && c < 0xE000)
+                  {
+                    // UTF-16 surrogate pair
+                    if (c < 0xDC00 && ::fread(utf8_, 2, 1, file_) == 1 && (static_cast<unsigned char>(utf8_[1]) & 0xFC) == 0xDC)
+                      c = 0x010000 - 0xDC00 + ((c - 0xD800) << 10) + (static_cast<unsigned char>(utf8_[0]) | static_cast<unsigned char>(utf8_[1]) << 8);
+                    else
+                      c = REFLEX_NONCHAR;
+                  }
+                  ulen_ = static_cast<unsigned short>(utf8(c, utf8_)); // always a short unsigned int
+                }
                 utfx_ = file_encoding::utf16le;
               }
             }
           }
           else if (utf8_[0] == '\xef' && utf8_[1] == '\xbb') // UTF-8 BOM EFBBXX?
           {
-            if (::fread(&utf8_[2], 1, 1, file_) == 1)
+            if (::fread(utf8_ + 2, 1, 1, file_) == 1)
             {
               ulen_ = 3;
               if (utf8_[2] == '\xbf') // UTF-8 BOM EFBBBF?
